package com.craftaro.ultimaterepairing.gui;

import com.craftaro.ultimaterepairing.UltimateRepairing;
import com.craftaro.ultimaterepairing.utils.Methods;
import com.craftaro.core.gui.Gui;
import com.craftaro.core.gui.GuiUtils;
import com.craftaro.core.utils.ItemUtils;
import com.craftaro.core.utils.TextUtils;
import com.craftaro.ultimaterepairing.repair.RepairType;
import com.craftaro.ultimaterepairing.settings.Settings;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import java.util.Arrays;

public class RepairGui extends Gui {

    private final Location anvil;
    private final Player player;
    private static final UltimateRepairing plugin = UltimateRepairing.getInstance();

    public static void newGui(Player player, Location anvil) {
        RepairType type = RepairType.EXPERIENCE;
        if (!type.hasPermission(player))
            type = type.getNext(player);
        if (type == null) {
            plugin.getLocale().getMessage("event.general.nopermission").sendPrefixedMessage(player);
            return;
        }
        plugin.getGuiManager().showGUI(player, new RepairGui(player, anvil, null, type));
    }

    private RepairGui(Player player, Location anvil, Gui gui, RepairType type) {
        super(gui);
        this.anvil = anvil;
        this.player = player;

        setRows(6);
        setTitle(plugin.getLocale().getMessage("interface.repair.title" + (hasRepairableItems() ? "" : "None")).getMessage());

        init(type);
    }

    protected void init(RepairType type) {
        if (inventory != null)
            inventory.clear();
        setActionForRange(0, 53, null);

        ItemStack glass1 = GuiUtils.getBorderItem(Settings.GLASS_TYPE_1.getMaterial());
        ItemStack glass2 = GuiUtils.getBorderItem(Settings.GLASS_TYPE_2.getMaterial());
        ItemStack glass3 = GuiUtils.getBorderItem(Settings.GLASS_TYPE_3.getMaterial());

        setDefaultItem(glass1);

        mirrorFill(0, 0, true, true, glass2);
        mirrorFill(0, 1, true, true, glass2);
        mirrorFill(0, 2, true, true, glass3);


        if (Arrays.stream(RepairType.values()).filter(p -> p.hasPermission(player)).count() > 1)
            setButton(4, GuiUtils.createButtonItem(type.getMaterial(),
                    type.getTitle(),
                    plugin.getLocale().getMessage("interface.repair.swap").getMessage()), (event) ->
                    init(type.getNext(player)));


        int i = 9;
        for (ItemStack item : getRepairableItems()) {
            ItemStack toRepair = item.clone();

            short durability = item.getDurability();

            final String itemName = TextUtils.formatText(ItemUtils.getItemName(item).replace("_", " "), true);
            if (type == RepairType.ECONOMY) {
                item = GuiUtils.createButtonItem(item,
                        plugin.getLocale().getMessage("interface.repair.item")
                                .processPlaceholder("ITEM", itemName).getMessage(),
                        plugin.getLocale().getMessage("interface.repair.ecolore").getMessage());
            } else if (type == RepairType.ITEM) {
                item = GuiUtils.createButtonItem(item,
                        plugin.getLocale().getMessage("interface.repair.item")
                                .processPlaceholder("ITEM", itemName).getMessage(),
                        plugin.getLocale().getMessage("interface.repair.itemlore")
                                .processPlaceholder("item", itemName).getMessage());
            } else if (type == RepairType.EXPERIENCE) {
                item = GuiUtils.createButtonItem(item,
                        plugin.getLocale().getMessage("interface.repair.item")
                                .processPlaceholder("ITEM", itemName).getMessage(),
                        plugin.getLocale().getMessage("interface.repair.xplore").getMessage());
            }
            item.setDurability(durability);

            int playerslot = player.getInventory().first(toRepair);
            setButton(i, item, (event) -> {
                exit();
                if (!player.getInventory().contains(toRepair)) {
                    plugin.getLocale().getMessage("event.repair.notfound").sendPrefixedMessage(player);
                    return;
                }
                player.getInventory().removeItem(toRepair);
                plugin.getRepairHandler().preRepair(toRepair, playerslot, player, type, anvil);
            });
            i++;
        }

        if (Settings.RAINBOW.getBoolean()) {
            for (int cell = 0; cell < rows * 9; ++cell) {
                if (getItem(cell) == null)
                    setItem(cell, GuiUtils.getBorderItem(Methods.getRainbowGlass()));
            }
        }
    }

    private ItemStack[] getRepairableItems() {
        return Arrays.stream(player.getInventory().getContents())
                .filter(item -> item != null && item.getDurability() > 0 && item.getMaxStackSize() == 1)
                .toArray(ItemStack[]::new);
    }

    private boolean hasRepairableItems() {
        return getRepairableItems().length > 0;
    }
}
